@@ -21,6 +21,16 @@ ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 
 #include "SDL_hints.h"
+#include "video.h"
+
+#include <the_Foundation/string.h>
+
+static iBool isTrue_(const char *value) {
+    return !iCmpStr(value, "1");
+}
 
 void SDL_SetHint(const char *id, const char *value) {
+    if (!iCmpStr(id, SDL_HINT_VIDEO_CURSES_SIMPLE_CHARACTERS)) {
+        isSimpleCharMode_Video = isTrue_(value);
+    }
 }
