@@ -237,18 +237,15 @@ static int processIncomingData_GmRequest_(iGmRequest *d, const iBlock *data) {
                           size_String(&resp->meta) - endPos - 2);
             remove_Block(&resp->meta.chars, endPos, iInvalidSize);
             /* Parse and remove the code. */
-            iRegExp *metaPattern = new_RegExp("^([0-9][0-9])(( )(.*))?", 0);
-            /* TODO: Empty <META> means no <SPACE>? Not according to the spec? */
+            iRegExp *metaPattern = new_RegExp("^([0-9][0-9])(( )(.*))?$", 0);
+            /* Empty <META> means no <SPACE>. */
             iRegExpMatch m;
             init_RegExpMatch(&m);
             int code = 0;
             if (matchString_RegExp(metaPattern, &resp->meta, &m)) {
                 code = atoi(capturedRange_RegExpMatch(&m, 1).start);
-                remove_Block(&resp->meta.chars,
-                             0,
-                             capturedRange_RegExpMatch(&m, 1).end -
-                                 constBegin_String(&resp->meta)); /* leave just the <META> */
-                trimStart_String(&resp->meta);
+                /* Leave just the <META>. */
+                set_String(&resp->meta, collect_String(captured_RegExpMatch(&m, 4)));
             }
             if (code == 0) {
                 clear_String(&resp->meta);
@@ -779,7 +776,7 @@ void setUrl_GmRequest(iGmRequest *d, const iString *url) {
        characters? Could be a server-side issue, e.g., if they're using a URL parser meant for
        the web. */
     /* Encode everything except already-percent encoded characters. */
-    iString *enc = urlEncodeExclude_String(&d->url, "%" URL_RESERVED_CHARS);
+    iString *enc = urlEncodeExclude_String(&d->url, URL_ENCODE_EXCLUDE_CHARS);
     /* Normalize empty paths to /. */ {
         iUrl parts;
         init_Url(&parts, enc);
@@ -1253,8 +1250,12 @@ void submit_GmRequest(iGmRequest *d) {
     }
     else {
         /* Normal Gemini request. */
-        setContent_TlsRequest(d->req,
-                              utf8_String(collectNewFormat_String("%s\r\n", cstr_String(&d->url))));
+        iString content;
+        init_String(&content);
+        format_String(&content, "%s\r\n", cstr_String(&d->url));
+        truncate_Block(&content.chars, 1024 /* maximum permitted URI length */ + 2 /* \r\n */);
+        setContent_TlsRequest(d->req, utf8_String(&content));
+        deinit_String(&content);
     }
     submit_TlsRequest(d->req);
 }
