@@ -23,12 +23,14 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 #include "media.h"
 #include "gmdocument.h"
 #include "gmrequest.h"
+#include "the_Foundation/block.h"
 #include "ui/window.h"
 #include "ui/paint.h" /* size_SDLTexture */
 #include "audio/player.h"
 #include "app.h"
 #include "stb_image.h"
-#include "stb_image_resize.h"
+#include "stb_image_resize2.h"
+#include "jpegxl.h" // LAGRANGE_ENABLE_JXL
 
 #if defined (LAGRANGE_ENABLE_WEBP)
 #   include <webp/decode.h>
@@ -41,6 +43,18 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 #include <SDL_render.h>
 #include <SDL_timer.h>
 
+struct Impl_Media {
+    iPtrArray items[max_MediaType];
+    /* TODO: Add a hash to quickly look up a link's media. */
+#if defined (LAGRANGE_ENABLE_JXL)
+    iJpegxl *jxl;
+#endif
+};
+
+iDefineTypeConstruction(Media)
+
+/*----------------------------------------------------------------------------------------------*/
+
 iDeclareType(GmMediaProps)
 
 struct Impl_GmMediaProps {
@@ -71,7 +85,9 @@ struct Impl_GmImage {
     iBlock        partialData; /* cleared when image is converted to texture */
     iInt2         size;
     size_t        numBytes;
-    SDL_Texture * texture;
+    SDL_Texture  *texture;
+    iBool         didFail;
+    iMedia       *media; /* backlink to owner */
 };
 
 void init_GmImage(iGmImage *d, const iBlock *data) {
@@ -80,6 +96,8 @@ void init_GmImage(iGmImage *d, const iBlock *data) {
     d->size     = zero_I2();
     d->numBytes = 0;
     d->texture  = NULL;
+    d->didFail  = iFalse;
+    d->media    = NULL;
 }
 
 void deinit_GmImage(iGmImage *d) {
@@ -100,7 +118,7 @@ static void applyImageStyle_(enum iImageStyle style, iInt2 size, uint8_t *imgDat
         iColor light = get_Color(tmParagraph_ColorId);
         if (hsl_Color(dark).lum > hsl_Color(light).lum) {
             iSwap(iColor, dark, light);
-        }        
+        }
         while (numPixels-- > 0) {
             iHSLColor hsl = hsl_Color((iColor){ pos[0], pos[1], pos[2], 255 });
             const float s = 1.0f - hsl.lum;
@@ -109,7 +127,7 @@ static void applyImageStyle_(enum iImageStyle style, iInt2 size, uint8_t *imgDat
             pos[1] = dark.g * s + light.g * t;
             pos[2] = dark.b * s + light.b * t;
             pos += 4;
-        }        
+        }
         return;
     }
     iColor colorize = (iColor){ 255, 255, 255, 255 };
@@ -133,19 +151,26 @@ static void applyImageStyle_(enum iImageStyle style, iInt2 size, uint8_t *imgDat
     }
 }
 
-void makeTexture_GmImage(iGmImage *d) {
+static iBool makeImageTexture_Media_(iMedia *media, iGmImage *d, iBool isPartial) {
     iBlock *data     = &d->partialData;
     d->numBytes      = size_Block(data);
     uint8_t *imgData = NULL;
-    if (cmp_String(&d->props.mime, "image/webp") == 0) {
+    iBool isNew      = iFalse;
+    if (!isPartial && equalMediaType_String(&d->props.mime, "image/webp")) {
 #if defined (LAGRANGE_ENABLE_WEBP)
         imgData = WebPDecodeRGBA(constData_Block(data), size_Block(data), &d->size.x, &d->size.y);
-#endif        
+#endif
     }
-    else {
+    else if (equalMediaType_String(&d->props.mime, "image/jxl")) {
+#if defined (LAGRANGE_ENABLE_JXL)
+        imgData = decodeImage_Jpegxl(media->jxl, d->props.linkId, data, isPartial, &d->size);
+#endif
+    }
+    else if (!isPartial) {
         imgData = stbi_load_from_memory(
             constData_Block(data), (int) size_Block(data), &d->size.x, &d->size.y, NULL, 4);
         if (!imgData) {
+            d->didFail = iTrue;
             fprintf(stderr, "[media] image load failed: %s\n", stbi_failure_reason());
         }
     }
@@ -154,7 +179,7 @@ void makeTexture_GmImage(iGmImage *d) {
         d->texture = NULL;
     }
     else {
-        applyImageStyle_(prefs_App()->imageStyle, d->size, imgData);        
+        applyImageStyle_(prefs_App()->imageStyle, d->size, imgData);
         /* TODO: Save some memory by checking if the alpha channel is actually in use. */
         iWindow *window  = get_Window();
         iInt2    texSize = d->size;
@@ -175,8 +200,11 @@ void makeTexture_GmImage(iGmImage *d) {
             }
             if (!isEqual_I2(scaled, d->size)) {
                 uint8_t *scaledImgData = malloc(scaled.x * scaled.y * 4);
-                stbir_resize_uint8(imgData, d->size.x, d->size.y, 4 * d->size.x,
-                                   scaledImgData, scaled.x, scaled.y, scaled.x * 4, 4);
+                stbir_resize_uint8_linear(imgData,
+                                          d->size.x, d->size.y, 4 * d->size.x,
+                                          scaledImgData,
+                                          scaled.x, scaled.y, scaled.x * 4,
+                                          STBIR_RGBA);
                 free(imgData);
                 imgData = scaledImgData;
                 texSize = scaled;
@@ -192,8 +220,12 @@ void makeTexture_GmImage(iGmImage *d) {
         d->texture = SDL_CreateTextureFromSurface(renderer_Window(window), surface);
         SDL_FreeSurface(surface);
         free(imgData);
+        isNew = iTrue;
     }
-    clear_Block(data);
+    if (!isPartial) {
+        clear_Block(data);
+    }
+    return isNew;
 }
 
 iDefineTypeConstructionArgs(GmImage, (const iBlock *data), data)
@@ -209,11 +241,15 @@ struct Impl_GmAudio {
 
 void init_GmAudio(iGmAudio *d) {
     init_GmMediaProps_(&d->props);
+#if defined (LAGRANGE_ENABLE_AUDIO)
     d->player = new_Player();
+#endif
 }
 
 void deinit_GmAudio(iGmAudio *d) {
+#if defined (LAGRANGE_ENABLE_AUDIO)
     delete_Player(d->player);
+#endif
     deinit_GmMediaProps_(&d->props);
 }
 
@@ -288,17 +324,13 @@ iDefineTypeConstruction(GmDownload)
 
 /*----------------------------------------------------------------------------------------------*/
 
-struct Impl_Media {
-    iPtrArray items[max_MediaType];
-    /* TODO: Add a hash to quickly look up a link's media. */
-};
-
-iDefineTypeConstruction(Media)
-
 void init_Media(iMedia *d) {
     iForIndices(i, d->items) {
         init_PtrArray(&d->items[i]);
     }
+#if defined (LAGRANGE_ENABLE_JXL)
+    d->jxl = new_Jpegxl();
+#endif
 }
 
 void deinit_Media(iMedia *d) {
@@ -306,6 +338,9 @@ void deinit_Media(iMedia *d) {
     iForIndices(i, d->items) {
         deinit_PtrArray(&d->items[i]);
     }
+#if defined (LAGRANGE_ENABLE_JXL)
+    delete_Jpegxl(d->jxl);
+#endif
 }
 
 void clear_Media(iMedia *d) {
@@ -321,6 +356,9 @@ void clear_Media(iMedia *d) {
     iForIndices(type, d->items) {
         clear_PtrArray(&d->items[type]);
     }
+#if defined (LAGRANGE_ENABLE_JXL)
+    clear_Jpegxl(d->jxl);
+#endif
 }
 
 size_t memorySize_Media(const iMedia *d) {
@@ -335,17 +373,19 @@ size_t memorySize_Media(const iMedia *d) {
             memSize += size_Block(&img->partialData);
         }
     }
+#if defined (LAGRANGE_ENABLE_AUDIO)
     iConstForEach(PtrArray, a, &d->items[audio_MediaType]) {
         const iGmAudio *audio = a.ptr;
         if (audio->player) {
             memSize += sourceDataSize_Player(audio->player);
         }
     }
+#endif
     iConstForEach(PtrArray, n, &d->items[download_MediaType]) {
         const iGmDownload *down = n.ptr;
         memSize += down->numBytes;
     }
-    return memSize; 
+    return memSize;
 }
 
 iBool setUrl_Media(iMedia *d, iGmLinkId linkId, enum iMediaType mediaType, const iString *url) {
@@ -383,18 +423,20 @@ iBool setData_Media(iMedia *d, iGmLinkId linkId, const iString *mime, const iBlo
         iGmImage *img;
         if (isDeleting) {
             take_PtrArray(&d->items[image_MediaType], existingIndex, (void **) &img);
+#if defined (LAGRANGE_ENABLE_JXL)
+            cancel_Jpegxl(d->jxl, linkId);
+#endif
             delete_GmImage(img);
         }
         else {
             img = at_PtrArray(&d->items[image_MediaType], existingIndex);
             iAssert(equal_String(&img->props.mime, mime)); /* MIME cannot change */
             set_Block(&img->partialData, data);
-            if (!isPartial) {
-                makeTexture_GmImage(img);
-            }
+            isNew = makeImageTexture_Media_(d, img, isPartial);
         }
     }
     else if (existing.type == audio_MediaType) {
+#if defined (LAGRANGE_ENABLE_AUDIO)
         iGmAudio *audio;
         if (isDeleting) {
             take_PtrArray(&d->items[audio_MediaType], existingIndex, (void **) &audio);
@@ -412,6 +454,7 @@ iBool setData_Media(iMedia *d, iGmLinkId linkId, const iString *mime, const iBlo
                 start_Player(audio->player);
             }
         }
+#endif
     }
     else if (existing.type == download_MediaType) {
         iGmDownload *dl;
@@ -437,16 +480,15 @@ iBool setData_Media(iMedia *d, iGmLinkId linkId, const iString *mime, const iBlo
         if (startsWith_String(mime, "image/")) {
             /* Copy the image to a texture. */
             iGmImage *img = new_GmImage(data);
+            img->media = d;
             img->props.linkId = linkId; /* TODO: use a hash? */
             img->props.isPermanent = !allowHide;
             set_String(&img->props.mime, mime);
             pushBack_PtrArray(&d->items[image_MediaType], img);
-            if (!isPartial) {
-                makeTexture_GmImage(img);
-            }
-            isNew = iTrue;
+            isNew = makeImageTexture_Media_(d, img, isPartial);
         }
         else if (startsWith_String(mime, "audio/")) {
+#if defined (LAGRANGE_ENABLE_AUDIO)
             iGmAudio *audio = new_GmAudio();
             audio->props.linkId = linkId; /* TODO: use a hash? */
             audio->props.isPermanent = !allowHide;
@@ -457,9 +499,11 @@ iBool setData_Media(iMedia *d, iGmLinkId linkId, const iString *mime, const iBlo
             }
             pushBack_PtrArray(&d->items[audio_MediaType], audio);
             /* Start playing right away. */
-            start_Player(audio->player);
-            postCommandf_App("media.player.started player:%p", audio->player);
+            if (start_Player(audio->player)) {
+                postCommandf_App("media.player.started player:%p", audio->player);
+            }
             isNew = iTrue;
+#endif /* LAGRANGE_ENABLE_AUDIO */
         }
     }
     return isNew;
@@ -506,6 +550,16 @@ iInt2 imageSize_Media(const iMedia *d, iMediaId imageId) {
     return zero_I2();
 }
 
+iBool imageFailed_Media(const iMedia *d, iMediaId imageId) {
+    iAssert(imageId.type == image_MediaType);
+    const size_t index = index_MediaId(imageId);
+    if (index < size_PtrArray(&d->items[image_MediaType])) {
+        const iGmImage *img = constAt_PtrArray(&d->items[image_MediaType], index);
+        return img->didFail;
+    }
+    return iTrue;
+}
+
 SDL_Texture *imageTexture_Media(const iMedia *d, iMediaId imageId) {
     iAssert(imageId.type == image_MediaType);
     const size_t index = index_MediaId(imageId);
@@ -574,12 +628,27 @@ iPlayer *audioPlayer_Media(const iMedia *d, iMediaId audioId) {
 }
 
 void pauseAllPlayers_Media(const iMedia *d, iBool setPaused) {
+#if defined (LAGRANGE_ENABLE_AUDIO)
     for (size_t i = 0; i < size_PtrArray(&d->items[audio_MediaType]); ++i) {
         const iGmAudio *audio = constAt_PtrArray(&d->items[audio_MediaType], i);
         if (audio->player) {
             setPaused_Player(audio->player, setPaused);
         }
     }
+#endif
+}
+
+size_t numActivePlayers_Media(const iMedia *d) {
+    size_t n = 0;
+#if defined (LAGRANGE_ENABLE_AUDIO)
+    for (size_t i = 0; i < size_PtrArray(&d->items[audio_MediaType]); ++i) {
+        const iGmAudio *audio = constAt_PtrArray(&d->items[audio_MediaType], i);
+        if (audio->player && isStarted_Player(audio->player) && !isPaused_Player(audio->player)) {
+            n++;
+        }
+    }
+#endif
+    return n;
 }
 
 void downloadStats_Media(const iMedia *d, iMediaId downloadId, const iString **path_out,
@@ -612,12 +681,16 @@ static void finished_MediaRequest_(iAnyObject *obj) {
 }
 
 void init_MediaRequest(iMediaRequest *d, iDocumentWidget *doc, unsigned int linkId,
-                       const iString *url, iBool enableFilters) {
+                       const iString *url, iBool enableFilters,
+                       const iGmIdentity *overrideDefaultIdentity) {
     d->doc    = doc;
     d->linkId = linkId;
     d->req    = new_GmRequest(certs_App());
     setUrl_GmRequest(d->req, url);
     enableFilters_GmRequest(d->req, enableFilters);
+    if (overrideDefaultIdentity) {
+        setIdentity_GmRequest(d->req, overrideDefaultIdentity);
+    }
     iConnect(GmRequest, d->req, updated, d, updated_MediaRequest_);
     iConnect(GmRequest, d->req, finished, d, finished_MediaRequest_);
     submit_GmRequest(d->req);
@@ -629,6 +702,19 @@ void deinit_MediaRequest(iMediaRequest *d) {
     iRelease(d->req);
 }
 
+void resubmitWithUrl_MediaRequest(iMediaRequest *d, const iString *url) {
+    iAssert(d->req);
+    iAssert(isFinished_GmRequest(d->req));
+    const iBool enableFilters = filtersEnabled_GmRequest(d->req);
+    deinit_MediaRequest(d); /* release request, disconnect audiences */
+    d->req = new_GmRequest(certs_App());
+    setUrl_GmRequest(d->req, url);
+    enableFilters_GmRequest(d->req, enableFilters);
+    iConnect(GmRequest, d->req, updated, d, updated_MediaRequest_);
+    iConnect(GmRequest, d->req, finished, d, finished_MediaRequest_);
+    submit_GmRequest(d->req);
+}
+
 iMediaRequest *newReused_MediaRequest(iDocumentWidget *doc, unsigned int linkId,
                                       iGmRequest *request) {
     iMediaRequest *d = new_Object(&Class_MediaRequest);
@@ -642,6 +728,6 @@ iMediaRequest *newReused_MediaRequest(iDocumentWidget *doc, unsigned int linkId,
 
 iDefineObjectConstructionArgs(MediaRequest,
                               (iDocumentWidget *doc, unsigned int linkId, const iString *url,
-                               iBool enableFilters),
-                              doc, linkId, url, enableFilters)
+                               iBool enableFilters, const iGmIdentity *overrideDefaultIdentity),
+                              doc, linkId, url, enableFilters, overrideDefaultIdentity)
 iDefineClass(MediaRequest)
