@@ -21,6 +21,7 @@ ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 
 #include "text.h"
+#include "defs.h"
 #include <SDL_version.h>
 
 iDeclareType(Font)
@@ -61,8 +62,8 @@ iLocalDef iBool isMeasuring_(enum iRunMode mode) {
     return (mode & modeMask_RunMode) == measure_RunMode;
 }
 
-static iRect runSimple_Font_(iFont *d, const iRunArgs *args) {
-    /* This function shapes text using a simplified, incomplete algorithm. It works for English 
+static void runSimple_Font_(iFont *d, const iRunArgs *args) {
+    /* This function shapes text using a simplified, incomplete algorithm. It works for English
        and other non-complex LTR scripts. Composed glyphs are not supported (must rely on text
        being in a pre-composed form). This algorithm is used if HarfBuzz is not available. */
     const iInt2 orig        = args->pos;
@@ -145,7 +146,11 @@ static iRect runSimple_Font_(iFont *d, const iRunArgs *args) {
                     iColor clr = get_Color(args->color);
                     ansiColors_Color(capturedRange_RegExpMatch(&m, 1),
                                      current_Text()->baseFgColorId,
-                                     none_ColorId, &clr, NULL);
+                                     none_ColorId,
+                                     iFalse,
+                                     &clr,
+                                     NULL,
+                                     NULL);
 #if defined (LAGRANGE_ENABLE_STB_TRUETYPE)
                     SDL_SetTextureColorMod(cache, clr.r, clr.g, clr.b);
 #endif
@@ -242,6 +247,9 @@ static iRect runSimple_Font_(iFont *d, const iRunArgs *args) {
                     const iColor clr = get_Color(colorNum);
 #if defined (LAGRANGE_ENABLE_STB_TRUETYPE)
                     SDL_SetTextureColorMod(cache, clr.r, clr.g, clr.b);
+#endif
+#if defined (SDL_SEAL_CURSES)
+                    SDL_SetRenderTextColor(render, clr.r, clr.g, clr.b);
 #endif
                     if (args->mode & fillBackground_RunMode) {
                         SDL_SetRenderDrawColor(render, clr.r, clr.g, clr.b, 0);
@@ -250,7 +258,7 @@ static iRect runSimple_Font_(iFont *d, const iRunArgs *args) {
                 prevCh = 0;
                 continue;
             }
-            if (isDefaultIgnorable_Char(ch) || isFitzpatrickType_Char(ch)) {
+            if (isControl_Char(ch)) {
                 continue;
             }
         }
@@ -278,7 +286,7 @@ static iRect runSimple_Font_(iFont *d, const iRunArgs *args) {
             iAssert(wrap);
             const char *wrapPos = currentPos;
             int advance = x1 - orig.x;
-            if (lastWordEnd && wrap->mode == word_WrapTextMode) {
+            if (lastWordEnd && lastWordEnd > args->text.start && wrap->mode == word_WrapTextMode) {
                 wrapPos = skipSpace_CStr(lastWordEnd); /* go back */
                 wrapPos = iMin(wrapPos, args->text.end);
                 advance = wrapAdvance;
@@ -298,7 +306,7 @@ static iRect runSimple_Font_(iFont *d, const iRunArgs *args) {
         }
         const int yLineMax = ypos + d->font.height;
         SDL_Rect dst = { x1 + glyph->d[hoff].x,
-                         ypos + glyph->font->baseline + glyph->d[hoff].y,
+                         ypos + glyph->font->font.baseline + glyph->d[hoff].y,
                          glyph->rect[hoff].size.x,
                          glyph->rect[hoff].size.y };
         if (glyph->font != d) {
@@ -324,7 +332,8 @@ static iRect runSimple_Font_(iFont *d, const iRunArgs *args) {
            is monospaced. Except with Japanese script, that's larger than the normal monospace. */
         const iBool useMonoAdvance = monoAdvance > 0; // && !isJapanese_FontId(fontId_Text_(glyph->font));
         const float advance = (useMonoAdvance && glyph->advance > 0 ? monoAdvance : glyph->advance);
-        if (!isMeasuring_(mode) && ch != 0x20 /* don't bother rendering spaces */) {
+        if (!isMeasuring_(mode) &&
+            (ch != 0x20 /* don't bother rendering spaces */ || (isTerminal_Platform() && dst.h == 2))) {
             if (useMonoAdvance && dst.w > advance && glyph->font != d && !isEmoji) {
                 /* Glyphs from a different font may need recentering to look better. */
                 dst.x -= (dst.w - advance) / 2;
@@ -357,6 +366,13 @@ static iRect runSimple_Font_(iFont *d, const iRunArgs *args) {
 #endif
 #if defined (SDL_SEAL_CURSES)
             SDL_RenderDrawUnicode(render, dst.x, dst.y, ch);
+            if (src.h == 2) {
+                /* "Big" font, used for titles: underline it. */
+                for (int ux = 0; ux < dst.w; ux++) {
+                    SDL_RenderDrawUnicode(render, dst.x + ux, dst.y + 1,
+                                          0x2500 /* box drawings light horizontal */);
+                }
+            }
 #endif
         }
         xpos += advance;
@@ -410,17 +426,17 @@ static iRect runSimple_Font_(iFont *d, const iRunArgs *args) {
     if (checkHitChar && wrap->hitChar == args->text.end) {
         wrap->hitAdvance_out = sub_I2(init_I2(xpos, ypos), orig);
     }
-    if (args->cursorAdvance_out) {
-        *args->cursorAdvance_out = sub_I2(init_I2(xpos, ypos), orig);
-    }
-    if (args->runAdvance_out) {
-        *args->runAdvance_out = xposMax - orig.x;
+    if (args->metrics_out) {
+        args->metrics_out->advance = sub_I2(init_I2(xpos, ypos), orig);
+        args->metrics_out->bounds = bounds;
     }
+//    if (args->runAdvance_out) {
+//        *args->runAdvance_out = xposMax - orig.x;
+//    }
 #if defined (SDL_SEAL_CURSES)
     if (mode & draw_RunMode) {
         SDL_SetRenderTextFillColor(render, 0, 0, 0, 0);
     }
 #endif
-    return bounds;
 }
 
